"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.NodeCollector = void 0;
const node_util_1 = require("node:util");
const utility_1 = require("@jfrog-pipelines/utility");
const logger_1 = require("@jfrog-pipelines/logger");
const NodeService_1 = require("./node/NodeService");
const NODE_TIMEOUT = 15;
const ZERO = 0;
const ONE_MINUTE_MILLISECONDS = 60000;
const ONE_DAY_MILLISECONDS = 86400000;
class NodeCollector {
    constructor(publisher) {
        this.logger = logger_1.LoggerFactory.getLogger('NodeCollector');
        this.nodeService = new NodeService_1.NodeService(publisher);
    }
    async deProvision(nodePool) {
        this.logger = logger_1.LoggerFactory.getLogger(`NodeCollector:${nodePool.id}${nodePool.name}`);
        try {
            this.logger.trace('Fetching nodes');
            const nodes = await this.getNodes(nodePool.id);
            this.logger.trace(`Active nodes are ${nodes.length}`);
            const idleNodes = this.nodeService.filterIdleNodes(nodePool, nodes);
            this.logger.trace(`idleNodes nodes are ${idleNodes.length}`);
            const nodesToDelete = this.computeDeleteState(nodePool, idleNodes);
            this.logger.trace(`Nodes to be deleted are ${nodesToDelete.length}`);
            if (nodePool.isCacheEnabled === true) {
                const nodesToCache = this.computeCachedState(nodePool, idleNodes, nodesToDelete);
                const cachedNodes = await this.applyCachedStateToNodes(nodesToCache);
                this.logger.info(`Cached ${cachedNodes.length} nodes`);
            }
            const deletedNodes = await this.applyDeleteStateToNodes(nodesToDelete);
            this.logger.info(`Deleted ${deletedNodes.length} nodes`);
        }
        catch (error) {
            this.logger.warn(`Failed to deProvision nodes for node pool ${nodePool.id} due to err ${(0, node_util_1.inspect)(error)}`);
        }
        finally {
            this.logger.trace(`Node collection completed`);
        }
    }
    async collectOrphanNodes() {
        try {
            this.logger.trace(`Fetching orphan nodes`);
            const nodes = await this.nodeService.getOrphanNodes();
            this.logger.trace(`Fetched ${nodes.length} orphan node/s`);
            const collectedNodes = await this.applyDeleteStateToNodes(nodes);
            this.logger.info(`Collected ${collectedNodes.length} orphan nodes.`);
            return collectedNodes.length;
        }
        catch (error) {
            this.logger.warn(`Failed to collectOrphan nodes due to err ${(0, node_util_1.inspect)(error)}`);
        }
        return ZERO;
    }
    async getNodes(poolID) {
        return this.nodeService.getAllNodes(poolID);
    }
    computeDeleteState(nodePool, nodes) {
        return nodes.filter((node) => {
            const shouldDelete = this.getNodeDeleteCondition(nodePool, node);
            return shouldDelete;
        });
    }
    getNodeDeleteCondition(nodePool, node) {
        const [currentDateInMS, currentDateOfMonth, currentMonthOfYear, currentYear, currentDayOfWeek] = this.getDateTuple();
        const nodeProvisionedAt = new Date(node.provisionedAt);
        const [provisionedAt, nodeProvisionedAtDate, nodeProvisionedAtMonth, nodeProvisionedAtYear] = this.getDateTuple(nodeProvisionedAt.toString());
        const CUTOFF_TIME = NODE_TIMEOUT * ONE_MINUTE_MILLISECONDS;
        const cutOffTimeForWaitingAt = currentDateInMS - CUTOFF_TIME;
        const statusLastUpdatedAt = new Date(node.statusLastUpdatedAt).getTime();
        const dayOfWeekMap = {
            sunday: 0,
            monday: 1,
            tuesday: 2,
            wednesday: 3,
            thursday: 4,
            friday: 5,
            saturday: 6
        };
        const daysPassedSinceProvision = (currentDateInMS - provisionedAt) / ONE_DAY_MILLISECONDS;
        if (node.statusCode === NodeService_1.NodeStatus.waiting && statusLastUpdatedAt < cutOffTimeForWaitingAt) {
            return true;
        }
        if (node.stopRequested) {
            return true;
        }
        if (node.statusCode === NodeService_1.NodeStatus.stopped) {
            return true;
        }
        if (node.statusCode === NodeService_1.NodeStatus.failure) {
            return true;
        }
        if (node.statusCode === NodeService_1.NodeStatus.deleted) {
            return true;
        }
        if (node.statusCode === NodeService_1.NodeStatus.processing) {
            return false;
        }
        if (this.isCachedEnabled(nodePool)) {
            if (this.checkIfNodeProvisionedToday(currentDateOfMonth, nodeProvisionedAtDate, currentMonthOfYear, nodeProvisionedAtMonth, currentYear, nodeProvisionedAtYear)) {
                return false;
            }
            if (this.isNodeCachedForStopIntervalDays(nodePool, daysPassedSinceProvision)) {
                return true;
            }
            const nodeStopDayOfWeek = nodePool.cacheSettingsPropertyBag.nodeStopDayOfWeek;
            if (currentDayOfWeek === dayOfWeekMap[nodeStopDayOfWeek]) {
                return true;
            }
        }
        else if (node.statusCode === NodeService_1.NodeStatus.cached) {
            return true;
        }
        else if (this.isSuccessNodePassedIdleInterval(nodePool, node, currentDateInMS, statusLastUpdatedAt)) {
            return true;
        }
        return false;
    }
    checkIfNodeProvisionedToday(currentDateOfMonth, nodeProvisionedAtDate, currentMonthOfYear, nodeProvisionedAtMonth, currentYear, nodeProvisionedAtYear) {
        return (currentDateOfMonth === nodeProvisionedAtDate &&
            currentMonthOfYear === nodeProvisionedAtMonth &&
            currentYear === nodeProvisionedAtYear);
    }
    isCachedEnabled(nodePool) {
        return nodePool.isCacheEnabled === true && !(0, utility_1.isEmpty)(nodePool.cacheSettingsPropertyBag);
    }
    isNodeCachedForStopIntervalDays(nodePool, daysPassedSinceProvision) {
        return (nodePool.cacheSettingsPropertyBag.nodeStopSetting === `nodeStopIntervalDays` &&
            daysPassedSinceProvision >= nodePool.cacheSettingsPropertyBag.nodeStopIntervalDays);
    }
    isSuccessNodePassedIdleInterval(nodePool, node, currentDateInMS, statusLastUpdatedAt) {
        return (nodePool.isCacheEnabled === false &&
            node.statusCode === NodeService_1.NodeStatus.success &&
            currentDateInMS - statusLastUpdatedAt >= nodePool.nodeIdleIntervalInMins * ONE_MINUTE_MILLISECONDS);
    }
    computeCachedState(nodePool, idleNodes, deletedNodes) {
        const ONE = 1;
        const nodes = new Map();
        for (let index = 0; index <= idleNodes.length - ONE; index++) {
            nodes.set(idleNodes[index].id, idleNodes[index]);
        }
        for (let index = 0; index <= deletedNodes.length - ONE; index++) {
            nodes.delete(idleNodes[index].id);
        }
        return [...nodes.values()];
    }
    async applyDeleteStateToNodes(nodes) {
        this.logger.trace(`Trying to deprovision ${nodes.length} nodes`);
        const deleteStatePromises = nodes.map((node) => {
            return this.nodeService.applyNodeToStoppedState(node);
        });
        return Promise.all(deleteStatePromises);
    }
    async applyCachedStateToNodes(nodes) {
        this.logger.trace(`Trying to cache ${nodes.length} nodes`);
        const cachedStatePromises = nodes.map((node) => {
            return this.nodeService.applyNodeToCachedState(node);
        });
        return Promise.all(cachedStatePromises);
    }
    getDateTuple(inputDate) {
        const currentDate = inputDate ? new Date(inputDate) : new Date();
        const time = currentDate.getTime();
        const date = currentDate.getDate();
        const month = currentDate.getMonth();
        const year = currentDate.getFullYear();
        const week = currentDate.getDay();
        return [time, date, month, year, week];
    }
}
exports.NodeCollector = NodeCollector;
//# sourceMappingURL=NodeCollector.js.map