"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.NodeProvisioner = void 0;
const node_util_1 = require("node:util");
const logger_1 = require("@jfrog-pipelines/logger");
const uuid_1 = require("uuid");
const configuration_1 = require("@jfrog-pipelines/configuration");
const BuildPlaneService_1 = require("./buildPlane/BuildPlaneService");
const NodeService_1 = require("./node/NodeService");
const ZERO = 0;
class NodeProvisioner {
    constructor(publisher) {
        this.logger = logger_1.LoggerFactory.getLogger('Provisioner');
        this.nodeService = new NodeService_1.NodeService(publisher);
        this.buildPlaneService = new BuildPlaneService_1.BuildPlaneService();
        this.properties = configuration_1.ConfigFactory.config;
    }
    async provision(nodePool) {
        this.logger = logger_1.LoggerFactory.getLogger(`NodeProvisioner:${nodePool.id}:${nodePool.name}`);
        try {
            this.logger.trace('Fetching nodes');
            const nodes = await this.getRunningNodes(nodePool.id);
            this.logger.trace(`Succesfully fetched ${nodes.length} node/s`);
            this.logger.trace('Fetching buildPlaneMessage count');
            const messageCount = await this.getBuildMessageCount(nodePool.id);
            this.logger.trace(`Current buildPlaneMessage count is ${messageCount}`);
            const availableNodes = this.nodeService.filterAvailableNodes(nodes);
            this.logger.trace(`Current availableNodes count is ${availableNodes.length}`);
            const requiredNodeCount = this.calculateRequiredNodes(availableNodes, messageCount);
            this.logger.trace(`Required nodes count is ${requiredNodeCount}`);
            if (requiredNodeCount <= ZERO) {
                this.logger.debug(`No new nodes are required to be provisioned`);
                return;
            }
            const cachedNodes = this.nodeService.filterCachedNodes(nodes);
            this.logger.trace(`Cached nodes count is ${cachedNodes.length}`);
            const activeNodes = this.nodeService.filterActiveNodes(nodes);
            this.logger.trace(`Active nodes count is ${activeNodes.length}`);
            const requiredNewNodes = this.getRequiredNewNodesCount(nodePool, activeNodes, cachedNodes, requiredNodeCount);
            this.logger.trace(`Required ${requiredNewNodes} more nodes to be created`);
            const startedCachedNpdes = await this.useCacheNodes(requiredNewNodes, cachedNodes);
            this.logger.info(`Started ${startedCachedNpdes.length} nodes`);
            if (requiredNewNodes > ZERO) {
                const provisionedNewNodes = await this.provisionActiveNodes(requiredNewNodes, nodePool);
                this.logger.info(`Created ${provisionedNewNodes.length} new nodes`);
            }
        }
        catch (error) {
            const message = `Failed to provision nodes for node pool id ${nodePool.id} due to ${(0, node_util_1.inspect)(error)}`;
            this.logger.warn(message);
            throw new Error(message);
        }
        finally {
            this.logger.trace(`Node provisioning completed.`);
        }
    }
    getRequiredNewNodesCount(nodePool, activeNodes, cachedNodes, requiredNodeCount) {
        const maxNodesToProvision = nodePool.numberOfNodes - activeNodes.length;
        const requiredNewNodes = Math.min(requiredNodeCount - cachedNodes.length, maxNodesToProvision);
        return requiredNewNodes;
    }
    async getRunningNodes(poolID) {
        return this.nodeService.getRunningNodes(poolID);
    }
    async getBuildMessageCount(poolID) {
        return this.buildPlaneService.getReadyMessageCount(poolID);
    }
    calculateRequiredNodes(availableNodes, messageCount) {
        const requiredCount = messageCount - availableNodes.length;
        return requiredCount;
    }
    async useCacheNodes(requiredCount, nodes) {
        const effectiveNodes = requiredCount < ZERO ? nodes.splice(-requiredCount) : nodes;
        this.logger.trace(`Starting ${effectiveNodes.length} cache nodes.`);
        const cachedNodePromises = effectiveNodes.map((node) => {
            return this.nodeService.applyNodeWaitingState(node);
        });
        return Promise.all(cachedNodePromises);
    }
    async provisionActiveNodes(count, nodePool) {
        const newRequiredNodes = Array.from({ length: count }).fill(null);
        const EIGHT = 8;
        const nodePoolProviderMetadataPropertyBag = { ...nodePool.providerMetadataPropertyBag };
        const newNodePromises = newRequiredNodes.map(() => {
            const uid = (0, uuid_1.v4)();
            const friendlyName = `node-${uid.slice(Math.max(ZERO, uid.length - EIGHT))}`;
            const sshPort = nodePoolProviderMetadataPropertyBag.sshPort ?? ZERO;
            const user = nodePoolProviderMetadataPropertyBag.sshUser ?? 'root';
            const buildPlaneVersion = this.properties.buildPlaneVersion ?? '';
            const nodeRequestBody = {
                projectId: nodePool.projectId,
                nodePoolId: nodePool.id,
                friendlyName: friendlyName,
                isOnDemand: true,
                statusCode: NodeService_1.NodeStatus.waiting,
                statusLastUpdatedAt: new Date(),
                createdBy: NodeService_1.nodeAccessorID,
                updatedBy: NodeService_1.nodeAccessorID,
                isAutoInitialized: false,
                stopRequested: false,
                intendedBuildPlaneVersion: buildPlaneVersion,
                projectIntegrationId: nodePool.projectIntegrationId,
                sshPort: sshPort,
                sshUser: user,
                systemPropertyBag: {},
                initPropertyBag: nodePool.initPropertyBag,
                providerMetadataPropertyBag: {
                    instanceSize: nodePoolProviderMetadataPropertyBag.instanceSize,
                    operatingSystem: nodePool.operatingSystem,
                    uniqueToken: (0, uuid_1.v4)()
                }
            };
            return this.nodeService.createNode(nodeRequestBody);
        });
        return Promise.all(newNodePromises);
    }
}
exports.NodeProvisioner = NodeProvisioner;
//# sourceMappingURL=NodeProvisioner.js.map